/**
 * \file AdjacencyList.cpp
 *
 * \date 17 janv. 2011
 * \author Bruno Belarte
 */

#ifndef ADJACENCY_LIST_CPP
#define ADJACENCY_LIST_CPP

#include "AdjacencyList.h"

class VertexIterator;

/**
 * \brief Default constructor.
 *
 * This constructor takes no parameters.
 */
template <typename VertexProperties, typename EdgeProperties>
AdjacencyList<VertexProperties, EdgeProperties>::AdjacencyList ()
{
}

/**
 * \brief Copy constructor.
 *
 * This constructor creates a copy of the AdjacencyList given in parameter.
 */
template <typename VertexProperties, typename EdgeProperties>
AdjacencyList<VertexProperties, EdgeProperties>::AdjacencyList(const AdjacencyList & other) :
	m_adjacency_list(other.m_adjacency_list),
	m_reverse_vertex_map(other.m_reverse_vertex_map)
{
}

/**
 * \brief Default destructor.
 */
template <typename VertexProperties, typename EdgeProperties>
AdjacencyList<VertexProperties, EdgeProperties>::~AdjacencyList ()
{
	clear();
}

/**
 * \brief Operator =
 * \param other : the graph to be affected with
 * \return A pointer to the current object.
 *
 *	This operator copies each vertex and their links between each others.
 */
template <typename VertexProperties, typename EdgeProperties>
AdjacencyList<VertexProperties, EdgeProperties> & AdjacencyList<VertexProperties, EdgeProperties>::operator =(const Self & other)
{
	AdjacencyList temp(other);

	swap(temp.m_adjacency_list, this->m_adjacency_list);
	swap(temp.m_reverse_vertex_map, this->m_reverse_vertex_map);

	return *this;
}

/**
 * \brief Add a vertex in the graph.
 *
 * \param prop : the value of the vertex to add
 * \return true if the vertex has been added to the graph.
 *
 * This method add a vertex in the graph if and only if the property is not already
 * in the graph. The vertex is adressed by is value.
 */
template <typename VertexProperties, typename EdgeProperties>
bool AdjacencyList<VertexProperties, EdgeProperties>::add_vertex (const VertexProperties & prop)
{
	if (!contains(prop))
	{
		VertexDescriptor v = boost::add_vertex(m_adjacency_list);
		properties(v) = prop;
		m_reverse_vertex_map.insert(std::pair<VertexProperties, VertexDescriptor>(prop, v));
		assert(vertices_count() == m_reverse_vertex_map.size());
		return true;
	}
	assert(vertices_count() == m_reverse_vertex_map.size());
	return false;
}

/**
 * \brief Remove a vertex from the graph.
 *
 * \param prop : the value of the vertex to remove
 * \return true if the vertex has been removed.
 *
 * This method remove the vertex if and only if the property is actually in the graph.
 * Keep in mind that removing a vertex will invalide all previously defined iterators.
 */
template <typename VertexProperties, typename EdgeProperties>
bool AdjacencyList<VertexProperties, EdgeProperties>::remove_vertex (const VertexProperties & prop)
{
	if (contains(prop))
	{
		boost::clear_vertex(m_reverse_vertex_map[prop],m_adjacency_list);
		boost::remove_vertex(m_reverse_vertex_map[prop], m_adjacency_list);
		m_reverse_vertex_map.erase(prop);
		assert(vertices_count() == m_reverse_vertex_map.size());
		return true;
	}
	assert(vertices_count() == m_reverse_vertex_map.size());
	return false;
}

/**
 * \brief Replace a vertex by another one.
 *
 * \param prop1 : the vertex to replace
 * \param prop2 : the new value of the vertex
 * \return true if the vartex has been properly replaced.
 *
 * This method replace the value of prop1 by prop2 if and only if the graph contains prop1
 * and doesn't cointain prop2. Otherwise, it does nothing. If prop1 is a value pointed by
 * an iterator, the iterator is still valid after this operation.
 */
template <typename VertexProperties, typename EdgeProperties>
bool AdjacencyList<VertexProperties, EdgeProperties>::replace_vertex (const VertexProperties & prop1, const VertexProperties & prop2)
{
	if (contains(prop1) && !contains(prop2))
	{
		ReverseVertexMapIterator iter = m_reverse_vertex_map.find(prop1);
		VertexDescriptor dprop1 = (*iter).second;
		properties(dprop1) = prop2;
		m_reverse_vertex_map.erase(iter);
		m_reverse_vertex_map.insert(std::pair<VertexProperties,VertexDescriptor>(prop2, dprop1));
		assert(vertices_count() == m_reverse_vertex_map.size());
		return true;
	}
	assert(vertices_count() == m_reverse_vertex_map.size());
	return false;
}

/**
 * \brief Return the number of vertices in the graph.
 *
 * \return the number of vertices in the graph.
 */
template <typename VertexProperties, typename EdgeProperties>
unsigned int AdjacencyList<VertexProperties, EdgeProperties>::vertices_count () const
{
	return num_vertices(m_adjacency_list);
}

/**
 * \brief Check if the graph contains a given vertex
 *
 * \param t : the value of the vertex to check
 * \return true if the graph contains the given vertex
 *
 * Cannot be const because the operator[] from map allow modification on the returned value.
 */
template <typename VertexProperties, typename EdgeProperties>
bool AdjacencyList<VertexProperties, EdgeProperties>::contains (const VertexProperties & t)
{
	return (m_reverse_vertex_map.find(t) != m_reverse_vertex_map.end());
}

/**
 * \brief Add edge 'edge' between vertices 'from' and 'to'.
 *
 * \param edge : the value of the edge to add
 * \param from : the value of the source vertex
 * \param to : the value of the target vertex
 * \return true if the edge has been properly added.
 *
 * The edge is added only if the current graph contains both vertices and if there is currently no edge between these two vertices.
 * As this graph is non oriented, the edge is added as an in and out edge for both of the vertices.
 */
template <typename VertexProperties, typename EdgeProperties>
bool AdjacencyList<VertexProperties, EdgeProperties>::add_edge (const EdgeProperties & edge, const VertexProperties & from, const VertexProperties & to)
{
	typename std::map<VertexProperties,VertexDescriptor>::iterator f = m_reverse_vertex_map.find(from);
	typename std::map<VertexProperties,VertexDescriptor>::iterator t = m_reverse_vertex_map.find(to);

	if ((f != m_reverse_vertex_map.end()) && (t != m_reverse_vertex_map.end()))
	{
		VertexDescriptor dfrom	= (*f).second;
		VertexDescriptor dto		= (*t).second;

		std::pair<EdgeDescriptor,bool> dedge = boost::add_edge(dfrom, dto, m_adjacency_list);

		if (dedge.second)
		{
			properties(dedge.first) = edge;
			assert(vertices_count() == m_reverse_vertex_map.size());
			return true;
		}
	}
	assert(vertices_count() == m_reverse_vertex_map.size());
	return false;
}

/**
 * \brief Remove an edge between two vertices.
 *
 * \param from : the value of the source vertex
 * \param to : the value of the target vertex
 * \return true if the edge has been properly removed.
 *
 * The edge is removed only if the current graph contains both vertices and if there is actually an edge between these two vertices.
 * As this graph is non oriented, the edge is removed as an in and out edge for both of the vertices.
 */
template <typename VertexProperties, typename EdgeProperties>
bool AdjacencyList<VertexProperties, EdgeProperties>::remove_edge (const VertexProperties & from, const VertexProperties & to)
{
	typename std::map<VertexProperties,VertexDescriptor>::iterator f = m_reverse_vertex_map.find(from);
	typename std::map<VertexProperties,VertexDescriptor>::iterator t = m_reverse_vertex_map.find(to);

	if ((f != m_reverse_vertex_map.end()) && (t != m_reverse_vertex_map.end()))
	{
		VertexDescriptor dfrom	= (*f).second;
		VertexDescriptor dto		= (*t).second;
		std::pair<EdgeDescriptor,bool> dedge = boost::edge(dfrom, dto, m_adjacency_list);

		if (dedge.second)
		{
			boost::remove_edge(dedge.first,m_adjacency_list);
			assert(vertices_count() == m_reverse_vertex_map.size());
			return true;
		}
	}
	assert(vertices_count() == m_reverse_vertex_map.size());
	return false;
}

/**
 * \brief Return the number of edges.
 *
 * \return the number of edges in the graph.
 */
template <typename VertexProperties, typename EdgeProperties>
unsigned int AdjacencyList<VertexProperties, EdgeProperties>::edges_count () const
{
	return num_edges(m_adjacency_list);
}

/**
 * \brief Remove all vertices and edges.
 */
template <typename VertexProperties, typename EdgeProperties>
void AdjacencyList<VertexProperties, EdgeProperties>::clear()
{
	m_adjacency_list.clear();
	m_reverse_vertex_map.clear();
	assert(vertices_count() == m_reverse_vertex_map.size());
}

/**
 * \brief Return an iterator pointing to the first vertex.
 * \return An iterator pointing to the first vertex.
 */
template <typename VertexProperties, typename EdgeProperties>
typename AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator
AdjacencyList<VertexProperties, EdgeProperties>::begin()
{
	VertexIterator v(this);
	v.begin();
	return v;
}

/**
 * \brief Return an iterator pointing to the lastvertex.
 * \return An iterator pointing to the last vertex.
 */
template <typename VertexProperties, typename EdgeProperties>
typename AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator
AdjacencyList<VertexProperties, EdgeProperties>::end()
{
	VertexIterator v(this);
	v.end();
	return v;
}

/**
 * \brief Return an iterator pointing to the first vertex.
 * \return An iterator pointing to the first vertex.
 */
template <typename VertexProperties, typename EdgeProperties>
typename AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeIterator
AdjacencyList<VertexProperties, EdgeProperties>::begin(const VertexProperties & prop)
{
	OutEdgeIterator v(this, prop);
	v.begin();
	return v;
}

/**
 * \brief Return an iterator pointing to the lastvertex.
 * \return An iterator pointing to the last vertex.
 */
template <typename VertexProperties, typename EdgeProperties>
typename AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeIterator
AdjacencyList<VertexProperties, EdgeProperties>::end(const VertexProperties & prop)
{
	OutEdgeIterator v(this, prop);
	v.end();
	return v;
}

/**
 * \brief Return a string describing the current graph.
 *
 * To use this method, the VertexProperties type should overload the << operator.
 */
template <typename VertexProperties, typename EdgeProperties>
std::string AdjacencyList<VertexProperties, EdgeProperties>::to_string ()
{
	std::string result("Adjacency list\n");
	std::ostringstream oss;

	VertexIterator v_begin	= this->begin();
	VertexIterator v_end	= this->end();

	for (v_begin; v_begin != v_end; ++v_begin)
	{
		oss << (*v_begin) << std::endl;

		OutEdgeIterator	begin = this->begin(*v_begin);
		OutEdgeIterator	end	=this->end(*v_begin);
		for (begin; begin != end; ++begin)
		{
			oss << "\t" << (*begin).second << " -> " << (*begin).first << std::endl;
		}
	}

	result += oss.str();

	return result;
}

/** Property access for vertices.*/
template <typename VertexProperties, typename EdgeProperties>
VertexProperties& AdjacencyList<VertexProperties, EdgeProperties>::properties(const VertexDescriptor& v)
{
	VertexPropertyMapType param = get(vertex_properties, m_adjacency_list);
	return param[v];
}

/** Property access for vertices (const version).*/
template <typename VertexProperties, typename EdgeProperties>
const VertexProperties& AdjacencyList<VertexProperties, EdgeProperties>::properties(const VertexDescriptor& v) const
{
	VertexPropertyMapConstType param = get(vertex_properties, m_adjacency_list);
	return param[v];
}

/** Property access for edges.*/
template <typename VertexProperties, typename EdgeProperties>
EdgeProperties& AdjacencyList<VertexProperties, EdgeProperties>::properties(const EdgeDescriptor& v)
{
	EdgePropertyMapType param = get(edge_properties, m_adjacency_list);
	return param[v];
}

/** Property access for edges (const version).*/
template <typename VertexProperties, typename EdgeProperties>
const EdgeProperties& AdjacencyList<VertexProperties, EdgeProperties>::properties(const EdgeDescriptor& v) const
{
	EdgePropertyMapConstType param = get(edge_properties, m_adjacency_list);
	return param[v];
}

#endif
