/**
 * \file AdjacencyList.h
 *
 * \date 17 janv. 2011
 * \author Bruno Belarte
 */

#ifndef ADJACENCYLIST_H_
#define ADJACENCYLIST_H_

// Boost
#include <boost/graph/adjacency_list.hpp>
#include <boost/graph/graph_traits.hpp>
#include <boost/graph/properties.hpp>

/** Definition of basic boost::graph properties. */
enum vertex_properties_t { vertex_properties };
enum edge_properties_t { edge_properties };
namespace boost
{
	BOOST_INSTALL_PROPERTY(vertex, properties);
	BOOST_INSTALL_PROPERTY(edge, properties);
}

/**
 * \class AdjacencyList
 * \brief Graph class represented as adjacency list.
 *
 * This class is basically a wrapper around boost::adjacency_list write for convenience.
 * It is templated over vertices properties and edges properties. It allows the user to
 * adress vertices by their value (through the use of operator==) instead of boost's vertex
 * descriptors, which are still use internally.
 */
template<typename VertexProperties, typename EdgeProperties>
class AdjacencyList
{
	// Typedefs
	public :
		typedef boost::adjacency_list
				<
					boost::setS,					// Allow only unique edge between two given vertices
					boost::listS,
					boost::undirectedS,			// Non oriented graph
					boost::property<vertex_properties_t, VertexProperties>,
					boost::property<edge_properties_t, EdgeProperties>
				> GraphType;

		typedef AdjacencyList<VertexProperties, EdgeProperties>	Self;
		typedef VertexProperties								VertexPropertiesType;
		typedef typename boost::graph_traits<GraphType>::vertex_descriptor	VertexDescriptor;
		typedef typename boost::graph_traits<GraphType>::edge_descriptor	EdgeDescriptor;
		typedef typename boost::graph_traits<GraphType>::vertex_iterator	VertexDescriptorIterator;
		typedef typename boost::graph_traits<GraphType>::edge_iterator		EdgeIterator;
		typedef std::pair<VertexProperties, EdgeProperties>					OutEdgeType;
		typedef typename boost::graph_traits<GraphType>::out_edge_iterator 	OutEdgeDescriptorIterator;

	private :
		typedef typename boost::property_map<GraphType, vertex_properties_t>::type 			VertexPropertyMapType;
		typedef typename boost::property_map<GraphType, vertex_properties_t>::const_type	VertexPropertyMapConstType;
		typedef typename boost::property_map<GraphType, edge_properties_t>::type			EdgePropertyMapType;
		typedef typename boost::property_map<GraphType, edge_properties_t>::const_type		EdgePropertyMapConstType;
		typedef std::map<VertexProperties,VertexDescriptor>									ReverseVertexMapType;
		typedef typename ReverseVertexMapType::iterator										ReverseVertexMapIterator;

	// Constructors, destructor and methods
	public:
		AdjacencyList();
		AdjacencyList(const Self & other);
		virtual ~AdjacencyList();

		Self & operator =(const Self & other);

		virtual bool			add_vertex		(const VertexProperties & prop);
		virtual bool			remove_vertex	(const VertexProperties & t);
		virtual bool			replace_vertex	(const VertexProperties & prop1, const VertexProperties & prop2);
		virtual unsigned int	vertices_count	() const;
		virtual bool			contains		(const VertexProperties & t);

		virtual bool			add_edge	(const EdgeProperties & edge, const VertexProperties & from, const VertexProperties & to);
		virtual bool			remove_edge	(const VertexProperties & from, const VertexProperties & to);
		virtual unsigned int	edges_count	() const;

		virtual void clear();

		// Iterator access
		class VertexIterator;
		class OutEdgeIterator;
		VertexIterator	begin ();
		VertexIterator	end ();
		OutEdgeIterator	begin (const VertexProperties & prop);
		OutEdgeIterator	end (const VertexProperties & prop);

		// Useful
		virtual std::string to_string();

	private :
		/** Property access for vertices.*/
      VertexProperties& 		properties(const VertexDescriptor& v);
		/** Property access for vertices (const version).*/
      const VertexProperties&	properties(const VertexDescriptor& v) const;
		/** Property access for edges.*/
      EdgeProperties&			properties(const EdgeDescriptor& v);
		/** Property access for edges (const version).*/
      const EdgeProperties&	properties(const EdgeDescriptor& v) const;

	// Attributes
	private:
		GraphType									m_adjacency_list;
		std::map<VertexProperties,VertexDescriptor>	m_reverse_vertex_map;

	public :
		/**
		 * \class VertexIterator
		 * \brief Vertex iterator class.
		 *
		 * This class allow to iterate through the vertices of an adjacency list.
		 * It is basically a wrapper around boost vertex_iterator, but allow
		 * access to the properties directly.
		 */
		class VertexIterator
		{
			public :
				// Typedefs
				typedef VertexIterator	Self;

				// Constructors
				VertexIterator();
				VertexIterator (AdjacencyList<VertexProperties, EdgeProperties> * graph);
				VertexIterator (VertexDescriptorIterator & iter);
				VertexIterator (const VertexIterator & other);

				Self &					operator= (const Self & other);
				const VertexProperties&	operator* ();
				Self &					operator++ ();
				bool					operator== (const Self & other) const;
				bool					operator!= (const Self & other) const;
				Self &					set (VertexProperties & prop);
				void					begin ();
				void					end ();

			private :
				AdjacencyList<VertexProperties, EdgeProperties> * m_Graph;
				VertexDescriptorIterator m_Iter;
				VertexDescriptorIterator m_Begin;
				VertexDescriptorIterator m_End;
		};	// end class VertexIterator

		/**
		 * \class OutEdgeIterator
		 * \brief Out edge iterator class.
		 *
		 * This class allow to iterate through a vertex out edges adjacency list.
		 * It is basically a wrapper around boost out_edge_iterator, but allow
		 * access to the properties directly.
		 */
		class OutEdgeIterator
		{
			public :
				// Typedefs
				typedef OutEdgeIterator	Self;

				// Constructors
				OutEdgeIterator();
				OutEdgeIterator (AdjacencyList<VertexProperties, EdgeProperties> * graph, const VertexProperties & prop);
				OutEdgeIterator (const OutEdgeIterator & other);

				Self &				operator= (const Self & other);
				const OutEdgeType	operator* ();
				Self &				operator++ ();
				bool				operator== (const Self & other) const;
				bool				operator!= (const Self & other) const;
				Self &				set (VertexProperties & vprop, EdgeProperties & eprop);
				void				begin ();
				void				end ();

			private :
				AdjacencyList<VertexProperties, EdgeProperties> * m_Graph;
				OutEdgeDescriptorIterator m_Iter;
				OutEdgeDescriptorIterator m_Begin;
				OutEdgeDescriptorIterator m_End;
		};	// end class OutEdgeIterator

};	// end class AdjacencyList

#include "AdjacencyList.cpp"
#include "AdjacencyListVertexIterator.cpp"
#include "AdjacencyListOutEdgeIterator.cpp"

#endif /* ADJACENCYLIST_H_ */
