/**
 * \brief Default constructor.
 */
template <typename VertexProperties, typename EdgeProperties>
AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeIterator::
OutEdgeIterator()
{
	m_Iter = m_Begin = m_End;
}

/**
 * \brief Constructor.
 *
 * \param graph : The adjacency list to iterate through.
 *
 * Construct an out edge iterator through the given graph.
 * Note that a valid property must be given in parameter.
 */
template <typename VertexProperties, typename EdgeProperties>
AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeIterator::
OutEdgeIterator (AdjacencyList<VertexProperties, EdgeProperties> * graph, const VertexProperties & prop)
{
	m_Graph = graph;
	assert(m_Graph->contains(prop));
	boost::tie(m_Begin,m_End) = boost::out_edges(m_Graph->m_reverse_vertex_map[prop], m_Graph->m_adjacency_list);
	m_Iter = m_Begin;
}

/**
 * \brief Copy constructor.
 * \param other : The OutEdgeIterator to copy.
 */
template <typename VertexProperties, typename EdgeProperties>
AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeIterator::
OutEdgeIterator (const OutEdgeIterator & other) :
	m_Graph(other.m_Graph),
	m_Iter(other.m_Iter),
	m_Begin(other.m_Begin),
	m_End(other.m_End)
{
}

/**
 * \brief operator=.
 * \param other : The OutEdgeIterator to copy.
 * \return The pointer to this object.
 */
template <typename VertexProperties, typename EdgeProperties>
typename AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeIterator &
AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeIterator::
operator= (const Self & other)
{
	OutEdgeIterator temp(other);

	swap(temp.m_Graph, this->m_Graph);
	swap(temp.m_Begin, this->m_Begin);
	swap(temp.m_End, this->m_End);
	swap(temp.m_Iter, this->m_Iter);

	return *this;
}

/**
 * \brief operator*
 * \return A std::pair containing the vertex and edge properties currently pointed by the iterator.
 *
 * Allow access to the properties of the vertex and edge currently pointed by the
 * iterator, but in read-only.
 */
template <typename VertexProperties, typename EdgeProperties>
const typename AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeType
AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeIterator::
operator* ()
{
	assert (m_Graph->contains(m_Graph->properties(boost::source(*m_Iter, m_Graph->m_adjacency_list))));
	assert (m_Graph->vertices_count() == m_Graph->m_reverse_vertex_map.size());
	assert (m_Graph->contains(m_Graph->properties(boost::target(*m_Iter, m_Graph->m_adjacency_list))));
	return std::make_pair(m_Graph->properties(boost::target(*m_Iter, m_Graph->m_adjacency_list)), m_Graph->properties(*m_Iter));
}

/**
 * \brief operator++
 * \return This pointer, with the new value of the iterator.
 */
template <typename VertexProperties, typename EdgeProperties>
typename AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeIterator &
AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeIterator::
operator++ ()
{
	++m_Iter;
	return *this;
}

/**
 * \brief operator==
 * \param other : The iterator to be compared with.
 * \return true if the two iterators point to the same value and have the same boundaries.
 *
 * Check if the inner pointers are the same.
 */
template <typename VertexProperties, typename EdgeProperties>
bool
AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeIterator::
operator== (const Self & other) const
{
	return (m_Begin == other.m_Begin) && (m_End == other.m_End) && (m_Iter == other.m_Iter);
}

/**
 * \brief operator!=
 * \param other : The iterator to be compared with.
 * \return true if the two iterators point to the different value or have the different boundaries.
 */
template <typename VertexProperties, typename EdgeProperties>
bool
AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeIterator::
operator!= (const Self & other) const
{
	return !(operator==(other));
}

/**
 * \brief Set the current pointer to the first vertex.
 */
template <typename VertexProperties, typename EdgeProperties>
void
AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeIterator::
begin()
{
	m_Iter = m_Begin;
}

/**
 * \brief Set the current pointer to the last vertex.
 */
template <typename VertexProperties, typename EdgeProperties>
void
AdjacencyList<VertexProperties, EdgeProperties>::OutEdgeIterator::
end()
{
	m_Iter = m_End;
}
