#ifndef ADJACENCYLISTITERATORS_CPP
#define ADJACENCYLISTITERATORS_CPP

#include "AdjacencyList.h"

/**
 * \brief Default constructor.
 */
template <typename VertexProperties, typename EdgeProperties>
AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator::
VertexIterator()
{
	m_Iter = m_Begin = m_End;
}

/**
 * \brief Constructor.
 *
 * \param graph : The adjacency list to iterate through.
 *
 * Construct an iterator through the given graph.
 */
template <typename VertexProperties, typename EdgeProperties>
AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator::
VertexIterator (AdjacencyList<VertexProperties, EdgeProperties> * graph)
{
	m_Graph = graph;
	boost::tie(m_Begin,m_End) = boost::vertices(m_Graph->m_adjacency_list);
	m_Iter = m_Begin;
}

/**
 * \brief Constructor.
 *
 * \param iter : The vertex descriptor of the vertex to point.
 *
 * This constructor is designed to be used inside the AdjacencyList class.
 * It allow to create an iterator which point to a given value instead of
 * the first vertex.
 */
template <typename VertexProperties, typename EdgeProperties>
AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator::
VertexIterator (VertexDescriptorIterator & iter) :
	m_Iter(iter)
{
}

/**
 * \brief Copy constructor.
 * \param other : The VertexIterator to copy.
 */
template <typename VertexProperties, typename EdgeProperties>
AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator::
VertexIterator (const VertexIterator & other) :
	m_Graph(other.m_Graph),
	m_Iter(other.m_Iter),
	m_Begin(other.m_Begin),
	m_End(other.m_End)
{
}

/**
 * \brief operator=.
 * \param other : The VertexIterator to copy.
 * \return The pointer to this object.
 */
template <typename VertexProperties, typename EdgeProperties>
typename AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator &
AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator::
operator= (const Self & other)
{
	VertexIterator temp(other);
	
	swap(temp.m_Graph, this->m_Graph);
	swap(temp.m_Begin, this->m_Begin);
	swap(temp.m_End, this->m_End);
	swap(temp.m_Iter, this->m_Iter);

	return *this;
}

/**
 * \brief operator*
 * \return The vertex properties currently pointed by the iterator.
 *
 * Allow access to the properties of the vertex currently pointed by the iterator,
 * but in read-only.
 */
template <typename VertexProperties, typename EdgeProperties>
const VertexProperties &
AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator::
operator* ()
{
	assert (m_Graph->contains(m_Graph->properties(*m_Iter)));
	assert (m_Graph->vertices_count() == m_Graph->m_reverse_vertex_map.size());
	return m_Graph->properties(*m_Iter);
}

/**
 * \brief operator++
 * \return This pointer, with the new value of the iterator.
 */
template <typename VertexProperties, typename EdgeProperties>
typename AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator &
AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator::
operator++ ()
{
	++m_Iter;
	return *this;
}

/**
 * \brief operator==
 * \param other : The iterator to be compared with.
 * \return true if the two iterators point to the same value and have the same boundaries.
 *
 * Check if the inner pointers are the same.
 */
template <typename VertexProperties, typename EdgeProperties>
bool
AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator::
operator== (const Self & other) const
{
	return (m_Begin == other.m_Begin) && (m_End == other.m_End) && (m_Iter == other.m_Iter);
}

/**
 * \brief operator!=
 * \param other : The iterator to be compared with.
 * \return true if the two iterators point to the different value or have the different boundaries.
 */
template <typename VertexProperties, typename EdgeProperties>
bool
AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator::
operator!= (const Self & other) const
{
	return !(operator==(other));
}

/**
 * \brief Change the value of the vertex pointed by the iterator.
 * \param prop : The new properties to set.
 * \return this pointer.
 *
 * This method call AdjacencyList::replace_vertex method.
 * This way, it allow only to set the value if not already present in
 * the adjacency list. This method does not invalidate the iterator.
 */
template <typename VertexProperties, typename EdgeProperties>
typename AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator &
AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator::
set(VertexProperties & prop)
{
	m_Graph->replace_vertex(this->operator*(), prop);
	return *this;
}

/**
 * \brief Set the current pointer to the first vertex.
 */
template <typename VertexProperties, typename EdgeProperties>
void
AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator::
begin()
{
	m_Iter = m_Begin; 
}

/**
 * \brief Set the current pointer to the last vertex.
 */
template <typename VertexProperties, typename EdgeProperties>
void
AdjacencyList<VertexProperties, EdgeProperties>::VertexIterator::
end()
{
	m_Iter = m_End;
}

#endif // ADJACENCYLISTITERATORS_CPP
