/**
 * \file Tree.h
 *
 * \date 30 mars 2011
 * \author Bruno Belarte
 */

#ifndef TREE_H_
#define TREE_H_

#include <vector>
#include <queue>
#include <stack>
#include <assert.h>
#include <boost/shared_ptr.hpp>
#include <boost/weak_ptr.hpp>

template <class T>
class TreeNode
{
	typedef TreeNode<T>				Node;
	typedef boost::weak_ptr<Node>	ParentPointer;
	typedef boost::shared_ptr<Node>	Pointer;

	public :
		T						data;
		ParentPointer			parent;
		std::vector<Pointer>	children;

		TreeNode (const T & t) :
			data(t),
			parent(ParentPointer())
		{}

		TreeNode (const T & t, Pointer n) :
			data(t),
			parent(ParentPointer(n))
		{}
};

template <class T>
class Tree
{
	typedef TreeNode<T>					NodeType;
	typedef boost::shared_ptr<NodeType>	NodePointer;
	typedef Tree<T>						Self;

	public :
		Tree();
		Tree(const T & t);

		class IteratorBase;
		class PreOrderIterator;
		class BreadthFirstIterator;

		typedef PreOrderIterator Iterator;

		IteratorBase				add (const T & t);
		template<class iter> iter	add (iter position, const T & t);
		IteratorBase				append (Self & other);
		template<class iter> iter	append (iter position, Self & other);

		template<class iter> iter	erase_children (iter position);
		template<class iter> iter	erase (iter position);
		void						clear ();

		template<class iter> iter	parent (iter position);
		template<class iter> iter	find (iter first, iter last, const T & t);

		NodeType *	getRoot ();
		Self 		setRoot (const T & t);

		PreOrderIterator		beginPre ();
		PreOrderIterator		endPre ();
		BreadthFirstIterator	beginBF ();
		BreadthFirstIterator	endBF ();

		class IteratorBase
		{
			public :
				IteratorBase ();
				IteratorBase (NodePointer);

				T & operator* () const;
				T * operator-> () const;

				NodePointer m_Node;
		};

		class PreOrderIterator : public IteratorBase
		{
			public:
				PreOrderIterator ();
				PreOrderIterator (NodePointer);
				PreOrderIterator (const IteratorBase &);

				bool	operator==(const PreOrderIterator &) const;
				bool	operator!=(const PreOrderIterator &) const;

				PreOrderIterator &	operator++ ();
				PreOrderIterator	operator++ (int);
				PreOrderIterator &	operator+= (unsigned int);

			private :
				std::stack<NodePointer> m_Stack;
		};

		class BreadthFirstIterator : public IteratorBase
		{
			public:
				BreadthFirstIterator ();
				BreadthFirstIterator (NodePointer);
				BreadthFirstIterator (const IteratorBase &);

				bool	operator==(const BreadthFirstIterator &) const;
				bool	operator!=(const BreadthFirstIterator &) const;

				BreadthFirstIterator &	operator++ ();
				BreadthFirstIterator	operator++ (int);
				BreadthFirstIterator &	operator+= (unsigned int);

			private :
				std::queue<NodePointer> m_Queue;
		};

	private :
		NodePointer m_Root;
};

#include "Tree.cxx"
#include "TreeIterators.cxx"

#endif /* TREE_H_ */
